//------------------------------------------------------------------------------
#ifndef ArraysH    //To avoid linker errors, define header's unique "name" so...
#define ArraysH    //...this header will not be compilled more than one time
//==============================================================================
#include <mem.h>     //Header containing declarations of memory routines
#include <stdlib.h>  //Standart library header
//------------------------------------------------------------------------------

//Version 1.0

/******************************************************************************\
//Template of simple container class. Use to store objects of any type, but
//preferably "data-classes", like integers, doubles, user-defined
//vector-containers and so on.
//Class TArray<...> is optimized for speed (referencing to elements does not
//cause out-of-range-checkings) and memory saving (size of occupyed memory is
//strictly proportional to contained items' quantity).
//NOTES:
//1) Elements are saved in aggregate array; changing amount of objects cause
//changing their's physical addresses (but not data, of course). So do not store
//pointers to any objects contained in array!
//2) Do not call constructor for array, which contains any items - they will
//lost and it cause memory leak! It is safe to call constructor for empty
//initialized array, but the best approach is to call destructor for it first.
//3) When increase the array size using ResizeTo function, no constructor is
//called for added items and their contents contain garbage. Besides, decreasing
//array size leading to items' losing does not cause calls to their destructors.
//4) Do not use auxilary classes also defined in this header.
\******************************************************************************/

//Auxilary classes defining location of items of some arrays:
template <class T> class TArrayRange;
template <class T> class TArrayRangesList;
//Main array class:
template <class T> class TArray;

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//Auxilary class containing record defining range of elements of some array
template <class T>
class TArrayRange{ public:
//Pointer to next range record
TArrayRange<T>* NextRecord;
//----- Range definition -----
T* RangeStartItem;
unsigned int ItemsInThisRange;
//----------------------------
//Constructor requires range to be defined
TArrayRange(const T* PointerToFirstRangeItem, unsigned int ItemsInRange){
 //Remember address of range beginning and quantity of items in range
 RangeStartItem = (T*)PointerToFirstRangeItem; //"const"-removing silly cast :)
 ItemsInThisRange = ItemsInRange;
 NextRecord = NULL; //Set pointer to next record to NULL
}
//Link existing record to this one (connect them)
inline void LinkTo(TArrayRange<T>* Record){
 NextRecord = Record; //Remember given address of next item
}
//Function creates new record after this one, links them and returns new pointer
inline TArrayRange<T>* AppendRecord(const T* PointerToFirstRangeItem,
                                    unsigned int ItemsInRange){
 //Create next record using given range definition
 NextRecord = new TArrayRange<T>(PointerToFirstRangeItem, ItemsInRange);
 //Return pointer to just-created record
 return NextRecord;
}
//Record will be deleted after copying range it defines; all next records will
//do the same - recursion used
inline void UseUp(T* *WritePosition){
 //Copy all items from range defined in this record to the specifyed location
 memcpy(*WritePosition,RangeStartItem,ItemsInThisRange*sizeof(T));
 //Move writing position by "ItemsInThisRange" items forward
 *WritePosition += ItemsInThisRange;
 //Temporarily copy pointer to next record and delete this one (as it was used)
 TArrayRange<T>* NextRecordPtr = NextRecord;
 delete this;
 //If next-record-pointer points to any record, make it to use itself up as well
 if(NextRecordPtr)NextRecordPtr->UseUp(WritePosition);
}
//Function deletes the whole chain of linked records using recursion
inline void Clear(){
 //Save pointer to next record until this one hasn't been deleted
 TArrayRange<T>* NextRecordPtr = NextRecord;
 delete this; //Delete this record
 //If next-record-pointer points to any record, make it to clear itself as well
 if(NextRecordPtr)NextRecordPtr->Clear();
}
};
//------------------------------------------------------------------------------

//Auxilary class containing list of records defining ranges of elements of some
//arrays; all records linked; see class' definition in the bottom of the file
template <class T>
class TArrayRangeList{
TArrayRange<T>* First;  //Pointer to first range-defining record in linked list
TArrayRange<T>* Last;   //Pointer to last record
unsigned int TotalItems;//Summ of elements in all ranges
//Allow class TArray to access private members of this class
friend class TArray<T>;
public:
//Constructor require first range to be defined
inline TArrayRangeList(const T* PointerToFirstRangeItem,
                       unsigned int ItemsInRange){
 //At first, summ of items is equal to the 1st record range' length
 TotalItems = ItemsInRange;
 First=new TArrayRange<T>(PointerToFirstRangeItem,ItemsInRange); //Create record
 Last = First; //As there is only one record, it is first and last equally
}
//Destructor does nothing: all range-records terminated automatically after used
inline ~TArrayRangeList(){
}
//Add new record in place before the first one in list
inline void PushRecord(const T* PointerToFirstRangeItem,
                       unsigned int ItemsInRange){
 //Summ quantity of items in this range with current
 TotalItems += ItemsInRange;
 //Save pointer to first record, create new in place of it and link them
 TArrayRange<T>* PointerToSecondRecord = First;
 First = new TArrayRange<T>(PointerToFirstRangeItem, ItemsInRange);
 First->LinkTo(PointerToSecondRecord);
}
//Append new record after the last one in list
inline void AddRecord(const T* PointerToFirstRangeItem,
                      unsigned int ItemsInRange){
 //Summ quantity of items in this range with current
 TotalItems += ItemsInRange;
 //Make last record to create new record and save returned pointer to new record
 //as new pointer to last record
 Last = Last->AppendRecord(PointerToFirstRangeItem, ItemsInRange);
}
// { Possible variants of using operator "+" }
//Using to merge two ranges-lists
inline TArrayRangeList<T> operator + (TArrayRangeList<T> Ranges){
 //Extend items' counter of this list by items in the rvalue list
 TotalItems += Ranges.TotalItems;
 //Link last record in this list to the first record in rvalue list
 Last->LinkTo(Ranges.First);
 //Now the last record in this list is the last record in rvalue list
 Last = Ranges.Last;
 return *this; //Finally return this list
}
//Single item also can be added to range list if it is treated as 1-item-array
inline TArrayRangeList<T> operator + (const T& SingleItem){
 TotalItems++; //Extend items counter by 1
 //Make last record to create new record for new item, treating it as 1-item
 //range and save returned pointer to new record as new pointer to last record
 Last = Last->AppendRecord(&SingleItem, 1);
 return *this; //Finally return this list
}
//This variant of operator + can be defined only after TArray definition
inline TArrayRangeList<T> operator + (const TArray<T>& Array);
//Operator "=" works only in case if lvalue range list contains only one range
//record and rvalue (range list or array) contains the same (or greater) number
//of items - it ensures that there is enough items to overwrite (!)
inline bool operator = (TArrayRangeList<T> Ranges){
 //Ensure that lvalue range list (this) contains only one item and there is
 //enough items in lvalue range allowed to overwrite
 if(First->NextRecord || TotalItems<Ranges.TotalItems){
  Ranges.First->Clear(); //Clear given range records' list...
  First->Clear();        //...and this list also; finally...
  return false;          //...indicate unsuccessfull result
 }
 //If everything OK, make current writing position to be start of range defined
 //by the first (and single) record of this ranges-list
 T* WritingPosition = First->RangeStartItem;
 //Make rvalue range records overwrite items of lvalue range
 Ranges.First->UseUp(&WritingPosition);
 //Release memory occupyed by this range and return positive result
 First->Clear();
 return true;
}
//This variant of operator = can be defined only after TArray definition
inline bool operator = (const TArray<T>& Array); //Defined in the bottom
//Function creates array with items of all ranges
inline void CreateArray(T* *NewItemLocation, unsigned int *NewItemCount){
 //Write quantity of items
 *NewItemCount = TotalItems;
 //Create new array large enough to fit all items
 *NewItemLocation = (T*)malloc(TotalItems*sizeof(T));
 //Create write-position-pointer which by default points to the start of array
 T* WritingPosition = *NewItemLocation;
 //Enforce first record to write items of range it defines to created array
 First->UseUp(&WritingPosition); //Because of recursion, all others do the same
}
//This variant of operator "==" checks two arrays' ranges for identity; both
//comparable ranges' lists should contain not more than 1 record, both of them
//defining ranges of the same length
inline bool operator == (TArrayRangeList<T> Ranges){
 //Ensure that both lists contain only one range record
 if(First->NextRecord || Ranges.First->NextRecord)return false;
 //Ensure equal quantity of items in both records (or lists - it doesn't matter)
 if(TotalItems != Ranges.TotalItems)return false;
 //If everything OK, check items ranges for identity
 bool Identity = ( memcmp( First->RangeStartItem, Ranges.First->RangeStartItem,
                           TotalItems * sizeof(T) ) == 0 );
 //Release memory occupyed by ranges' lists
 First->Clear();
 Ranges.First->Clear();
 return Identity;
}
//This variant of operator "==" checks identity of contents of given array and
//range defined by single record of this list; this range' lists should contain
//not more than 1 record, defining range of the length equal to array's length
inline bool operator == (TArray<T>& ComparableArray); //Defined in the bottom
};
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//Main array class
template <class T>
class TArray{
//------------------------------------------------------------------------------
unsigned int ItemsCount; //How many items of class T this array contains
T *Items;                //Pointer to memory location where items are stored
//------------------------------------------------------------------------------

//Allow class TArrayRangeList to access private members of this class
friend class TArrayRangeList<T>;
//------------------------------------------------------------------------------
public:

//Default constructor
inline TArray(){
 ItemsCount = 0; //Set items counter to 0
 Items = NULL;   //Set pointer to items' array to NULL by default
}
//------------------------------------------------------------------------------

//Copy-constructor
inline TArray(const TArray<T>& SourceArray){
 ItemsCount = SourceArray.ItemsCount;      //Copy items' counter
 Items = (T*)malloc(sizeof(T)*ItemsCount); //Allocate individual memory chunk
 memcpy(Items,SourceArray.Items,ItemsCount*sizeof(T)); //Copy items to it
 Beep();
}
//------------------------------------------------------------------------------

//Constructor for creating array from items defined in range records' list
inline TArray(TArrayRangeList<T> ArraysRanges){
 ArraysRanges.CreateArray(&Items,&ItemsCount); //Create new array
}
//------------------------------------------------------------------------------

//This constructor creates array of the specifyed size at initialization
inline TArray(const unsigned int DefaultItemsQuantity){
 ItemsCount = DefaultItemsQuantity;        //Save requested length' value
 Items = (T*)malloc(sizeof(T)*ItemsCount); //Allocate memory
}
//------------------------------------------------------------------------------

//Function resizes array to specifyed number of elements
inline void ResizeTo(unsigned int NewQuantity){
 T* Items=(T*)malloc(sizeof(T)*NewQuantity); //Create new memory location
 //Copy allowed quantity of items from old location to newly created; the
 //allowed quantity is size of shortest array (old one or new one)
 memcpy(NewItems, Items,
        ((NewQuantity<ItemsCount)?(NewQuantity):(ItemsCount)) * sizeof(T));
 free(Items);             //Release old memory location
 Items = NewItems;        //Assign pointer to newly created memory location
 ItemsCount = NewQuantity;//Remember new item's quantity
}
//------------------------------------------------------------------------------

//Function resizes array by specifyed number of elements (crop array/add empty)
inline void ResizeBy(signed int ItemsQuantityChange){
 //Create temporary variables defining newly created array
 unsigned int NewItemsCount=ItemsCount+ItemsQuantityChange; //Count new quantity
 T* NewItems=(T*)malloc(sizeof(T)*NewItemsCount); //Create new memory location
 //Copy allowed quantity of items from old location to newly created; the
 //allowed quantity is size of shortest array (old one or new one)
 memcpy(NewItems, Items,
        ((NewItemsCount<ItemsCount)?(NewItemsCount):(ItemsCount)) * sizeof(T));
 free(Items);               //Release old memory location
 Items = NewItems;          //Assign pointer to newly created memory location
 ItemsCount = NewItemsCount;//Remember new item's quantity
}
//------------------------------------------------------------------------------  

//Function returns current length of array
inline unsigned int TotalItems(){
 return ItemsCount;
}
//------------------------------------------------------------------------------

//Get real address of items' array in memory
inline T* ItemsAddress(){
 return Items;
}
//------------------------------------------------------------------------------

//Function calculates size of array in bytes
inline unsigned int ByteSize(){
 return ItemsCount * sizeof(T);
}
//------------------------------------------------------------------------------

//Operator [] used for referencing to individual elements
inline T& operator [] (unsigned int ItemN){
 return Items[ItemN];
}
//------------------------------------------------------------------------------  

//Function returns last element in array
inline T& LastItem(){
 return Items[ItemsCount-1];
}
//------------------------------------------------------------------------------

//Operator () used for referencing to range of elements which starts from
//"RangeStartItem" and contains "ItemsInRange" items
inline TArrayRangeList<T> operator () (unsigned int RangeStartItem,
                                       unsigned int ItemsInRange){
 return TArrayRangeList<T>(&Items[RangeStartItem],ItemsInRange);
}
//------------------------------------------------------------------------------ 

//Operator () used for referencing to range of elements which starts from
//"RangeStartItem" and contains all array's items after this one
inline TArrayRangeList<T> operator () (unsigned int RangeStartItem){
 return TArrayRangeList<T>(&Items[RangeStartItem],ItemsCount-RangeStartItem);
}
//------------------------------------------------------------------------------

//Define assigning operator, to handle proper items' copying
inline void operator = (const TArray<T>& SourceArray){
 free(Items); //Release memory occupyed by this array
 ItemsCount = SourceArray.ItemsCount;      //Copy items' counter
 Items = (T*)malloc(sizeof(T)*ItemsCount); //Create new memory location
 memcpy(Items,SourceArray.Items,ItemsCount*sizeof(T)); //Copy items to it
}
//------------------------------------------------------------------------------

//Define assigning operator for copying items using range list
void operator = (TArrayRangeList<T> ArraysRanges){
 //Create temporary containers
 unsigned int NewItemsCount = 0;
 T* NewItems = NULL;
 ArraysRanges.CreateArray(&NewItems,&NewItemsCount); //Create new array
 free(Items); //Release old items
 //Assign pointer to newly created array and update items counter
 Items = NewItems;
 ItemsCount = NewItemsCount;
}
//------------------------------------------------------------------------------ 

//Define assigning operator for resizing array to single-item-array
void operator = (T& SingleItem){
 free(Items);                   //Release memory occupyed by this array
 ItemsCount = 1;                //Set items counter to 1
 Items = (T*)malloc(sizeof(T)); //Create new memory location for single element
 Items[0] = SingleItem;         //Copy provided item to it
}
//------------------------------------------------------------------------------

//Append item to the end of array
inline void operator += (T NewItem){
 //Create new bigger memory location
 T* NewItems=(T*)malloc(sizeof(T)*(ItemsCount+1));
 //Copy items to newly created array
 memcpy(NewItems,Items,ItemsCount*sizeof(T));
 NewItems[ItemsCount]=NewItem; //Copy new item to last position in array
 //Release old memory location and assign pointer to newly created array
 free(Items);
 Items = NewItems;
 ItemsCount++; //Finally, extend items' counter by 1
}
//------------------------------------------------------------------------------

//Append items of another array to the end of this array
inline void operator += (const TArray<T>& SourceArray){
 //Create new memory location large enough to adopt all items from both arrays
 T* NewItems=(T*)malloc(sizeof(T)*(ItemsCount+SourceArray.ItemsCount));
 //Copy items to newly created array
 memcpy(NewItems,Items,ItemsCount*sizeof(T));
 //Add items from given array right after last item of this array
 memcpy(&NewItems[ItemsCount], SourceArray.Items,
        SourceArray.ItemsCount*sizeof(T));
 //Release old memory location and assign pointer to newly created array
 free(Items);
 Items = NewItems;
 ItemsCount += SourceArray.ItemsCount; //Update items' counter
}
//------------------------------------------------------------------------------

//Append items of another arrays' ranges to the end of this array
inline void operator += (TArrayRangeList<T> ArraysRanges){
 //Create record about this array and push it in front of others in list
 ArraysRanges.PushRecord(Items,ItemsCount);
 //Create temporary containers
 unsigned int NewItemsCount = 0;
 T* NewItems = NULL;
 ArraysRanges.CreateArray(&NewItems,&NewItemsCount); //Create new array
 free(Items); //Release old items
 //Assign pointer to newly created array and update items counter
 Items = NewItems;
 ItemsCount = NewItemsCount;
}
//------------------------------------------------------------------------------

//Operator + creates range record defining all array's elements, pushes this
//record in front of others in rvalue range list and returns it
inline TArrayRangeList<T> operator + (TArrayRangeList<T> ArraysRanges){
 //Push record defining this array in front of others and return resulted list
 ArraysRanges.PushRecord(Items, ItemsCount);
 return ArraysRanges;
}
//------------------------------------------------------------------------------

//Operator + creates range record defining all array's elements, adds record
//defining all elements of rvalue record, and returns resulting ranges-list
inline TArrayRangeList<T> operator + (const TArray<T>& AnotherArray){
 //Create record about this array
 TArrayRangeList<T> Ranges(Items,ItemsCount);
 //Add record defining another array and return resulted list
 Ranges.AddRecord(AnotherArray.Items, AnotherArray.ItemsCount);
 return Ranges;
}
//------------------------------------------------------------------------------

//Operator + creates range record defining all elements of this array and adds
//record defining location of single item provided, treating it as array
inline TArrayRangeList<T> operator + (const T& SingleItem){
 //Create record about this array
 TArrayRangeList<T> Ranges(Items,ItemsCount);
 //Add record about this single item and return resulted list
 Ranges.AddRecord(&SingleItem, 1);
 return Ranges;
}     
//------------------------------------------------------------------------------

//Operator == compares datum of two arrays
inline bool operator == (const TArray<T>& ArrayToCompare){
 //If arrays differs at length, return notequality answer
 if(ItemsCount != ArrayToCompare.ItemsCount)return false;
 //Otherwise check their datum for identity
 return (memcmp(Items,ArrayToCompare.Items,ItemsCount*sizeof(T)) == 0);
}
//------------------------------------------------------------------------------

//This variant of operator "==" checks array's contents for identity with ones
//of range defined by the single record of the rvalue ranges list; array should
//contain the same amount of items as single range defined in list
inline bool operator == (TArrayRangeList<T> SingleRangeList){
 //Ensure that list contains only one range record
 if(SingleRangeList.First->NextRecord)return false;
 //Ensure equal quantity of items
 if(ItemsCount != SingleRangeList.TotalItems)return false;
 //If everything OK, check contents for identity
 bool Identity = ( memcmp( Items, SingleRangeList.First->RangeStartItem,
                           ItemsCount * sizeof(T) ) == 0 );
 //Release memory occupyed by ranges' list
 SingleRangeList.First->Clear();
 return Identity;
}
//------------------------------------------------------------------------------

//Destructor
inline ~TArray(){
 free(Items);  //Release used memory
}
//------------------------------------------------------------------------------

//Function creates and returns range record for specifyed single item
inline TArrayRangeList<T> ItemToRange(const T& SingleItem){
 return TArrayRangeList<T>(&SingleItem,1);
}
//------------------------------------------------------------------------------

//Function cuts item with specyfied index from array
inline void Cut(unsigned int ItemID){
 ItemsCount--;                                 //Count new items quantity
 T* NewItems=(T*)malloc(ItemsCount*sizeof(T)); //Create new memory location
 //Copy items left from cropped one to newly created array
 memcpy(NewItems,Items,ItemID*sizeof(T));
 //Copy items right from cropped one to newly created array
 memcpy(&NewItems[ItemID],&Items[ItemID+1],(ItemsCount-ItemID)*sizeof(T));
 //Release old memory location and assign pointer to newly created array
 free(Items);
 Items = NewItems;
}
//------------------------------------------------------------------------------

//Function cuts specifyed quantity of items starting from one which index given
inline void Cut(unsigned int FirstItemID, unsigned int HowManyItemsToCut){
 ItemsCount -= HowManyItemsToCut;             //Count new items quantity
 T* NewItems=(T*)malloc(ItemsCount*sizeof(T));//Create new memory location
 //Copy items left from cropped area to newly created array
 memcpy(NewItems,Items,FirstItemID*sizeof(T));
 //Copy items right from cropped area to newly created array
 memcpy(&NewItems[FirstItemID], &Items[FirstItemID+HowManyItemsToCut],
        (ItemsCount-FirstItemID)*sizeof(T));
 //Release old memory location and assign pointer to newly created array
 free(Items);
 Items = NewItems;
}
//------------------------------------------------------------------------------

//Insert given element in specyfied position
inline void Insert(unsigned int InsertPosition, T ItemToInsert){
 ItemsCount++;                                //Count new items quantity
 T* NewItems=(T*)malloc(ItemsCount*sizeof(T));//Create new memory location
 //Copy items left from insertion position to newly created array
 memcpy(NewItems,Items,InsertPosition*sizeof(T));
 //Copy given item to specifyed position
 NewItems[InsertPosition] = ItemToInsert;
 //Copy items right from insertion position to newly created array
 memcpy(&NewItems[InsertPosition+1], &Items[InsertPosition],
        (ItemsCount-InsertPosition-1)*sizeof(T));
 //Release old memory location and assign pointer to newly created array
 free(Items);
 Items = NewItems;
}
//------------------------------------------------------------------------------

//Insert all items of given array in specyfied position
inline void Insert(unsigned int InsertPosition, TArray<T>& ArrayToInsert){
 //Count new items quantity and create new array
 unsigned int NewItemsCount = ItemsCount + ArrayToInsert.ItemsCount;
 T* NewItems=(T*)malloc(NewItemsCount*sizeof(T));
 //Copy items left from insertion position to newly created array
 memcpy(NewItems,Items,InsertPosition*sizeof(T));
 //Copy items from given given array to new one starting from specifyed position
 memcpy(&NewItems[InsertPosition],ArrayToInsert.Items,
        ArrayToInsert.ItemsCount*sizeof(T));
 //Copy items right from insertion position to newly created array
 memcpy(&NewItems[InsertPosition+ArrayToInsert.ItemsCount],
        &Items[InsertPosition],
        (ItemsCount-InsertPosition)*sizeof(T));
 //Release old memory location and assign pointer to newly created array
 free(Items);
 Items = NewItems;
 ItemsCount = NewItemsCount; //Remember new items quantity
}
//------------------------------------------------------------------------------

//Insert all items of given arrays ranges in specyfied position
inline void Insert(unsigned int InsertPosition,TArrayRangeList<T> ArraysRanges){
 //Create records about two parts of this array and add them to list
 ArraysRanges.PushRecord(Items,InsertPosition);
 ArraysRanges.AddRecord(&Items[InsertPosition],ItemsCount-InsertPosition);
 //Create temporary containers
 unsigned int NewItemsCount = 0;
 T* NewItems = NULL;
 ArraysRanges.CreateArray(&NewItems,&NewItemsCount); //Create new array
 //Release old memory location and assign pointer to newly created array
 free(Items);
 Items = NewItems;
 ItemsCount = NewItemsCount; //Remember new items quantity
}
//------------------------------------------------------------------------------

};
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


/******************************************************************************\
|----------- Definitions of some of TArrayRangeList class functions -----------|
\******************************************************************************/
template <class T>
inline TArrayRangeList<T>
TArrayRangeList<T>::operator + (const TArray<T>& Array){
 //Summ items counters
 TotalItems += Array.ItemsCount;
 //Add new record defining range including all items of given array
 Last = Last->AppendRecord(Array.Items, Array.ItemsCount);
 return *this;
}
//------------------------------------------------------------------------------
template <class T>
inline bool
TArrayRangeList<T>::operator = (const TArray<T>& Array){
 //Ensure that lvalue range list (this) contains only one item and there is
 //enough items in lvalue array allowed to be overwritten
 if(First->NextRecord || TotalItems<Array.ItemsCount){
  First->Clear();        //Clear this list and...
  return false;          //...indicate unsuccessfull result
 }
 //If OK, copy all items of rvalue array to location specifyed by this range
 memcpy(First->RangeStartItem,Array.Items,Array.ItemsCount*sizeof(T));
 //Release memory occupyed by this range and return positive result
 First->Clear();
 return true;
}
//------------------------------------------------------------------------------
template <class T>
inline bool
TArrayRangeList<T>::operator == (TArray<T>& ComparableArray){
 //Ensure that list contain only one range record
 if(First->NextRecord)return false;
 //Ensure equal quantity of items in both single-record list and array
 if(TotalItems != ComparableArray.ItemsCount)return false;
 //If everything OK, check contents for identity
 bool Identity = ( memcmp( First->RangeStartItem, ComparableArray.Items,
                           TotalItems * sizeof(T) ) == 0 );
 //Release memory occupyed by ranges' list
 First->Clear();
 return Identity;
}
//------------------------------------------------------------------------------


//==============================================================================
#endif //End of header
